#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

varying vec2 textureCoordinate;

uniform float exposure;

vec4 saturation(vec4 inp, float saturation) {
    vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);
    float luminance = dot(inp.rgb, luminanceWeighting);
    vec3 greyScaleColor = vec3(luminance);
    return vec4(mix(greyScaleColor, inp.rgb, saturation), inp.w);
}

vec4 exposure_f(vec4 inp, float exp) {
    vec3 expow = vec3(pow(2.0, exp));
    return vec4(inp.rgb * expow, inp.a);
}

float overlay_c(float bc, float oc, float ba, float oa) {
    float ca;
    if (2.0 * bc < ba) {
        ca = 2.0 * oc * bc + oc * (1.0 - ba) + bc * (1.0 - oa);
    } else {
        ca = oa * ba - 2.0 * (ba - bc) * (oa - oc) + oc * (1.0 - ba) + bc * (1.0 - oa);
    }
    return ca;
}

vec4 overlay(vec4 base, vec4 overlay) {
    float ra = overlay_c(base.r, overlay.r, base.a, overlay.a);
    float ga = overlay_c(base.g, overlay.g, base.a, overlay.a);
    float ba = overlay_c(base.b, overlay.b, base.a, overlay.a);
    return vec4(ra, ga, ba, 1.0);
}

void main(void) {
    vec4 base = texture2D(inputImageTexture, textureCoordinate);
    vec4 destination = texture2D(inputImageTexture2, textureCoordinate);

    vec4 outColor = saturation(base, 0.0);// Desaturation
    outColor = exposure_f(outColor, exposure);// Exposure
    outColor = overlay(outColor, destination);// Overlay

    gl_FragColor = mix(base, outColor, destination.a);
}