precision highp float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform lowp float opacity;
uniform int drawing;

const mediump vec3 filterColor = vec3(0.7098, 0.3411, 0.4901);
const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

lowp vec4 monochromaticColor(vec4 textureColor){
    float luminance = dot(textureColor.rgb, luminanceWeighting);
    lowp vec4 desat = vec4(vec3(luminance), 1.0);
    //overlay
    lowp vec4 outputColor = vec4(
    (desat.r < 0.5 ? (2.0 * desat.r * filterColor.r) : (1.0 - 2.0 * (1.0 - desat.r) * (1.0 - filterColor.r))),
    (desat.g < 0.5 ? (2.0 * desat.g * filterColor.g) : (1.0 - 2.0 * (1.0 - desat.g) * (1.0 - filterColor.g))),
    (desat.b < 0.5 ? (2.0 * desat.b * filterColor.b) : (1.0 - 2.0 * (1.0 - desat.b) * (1.0 - filterColor.b))),
    1.0
    );
    return outputColor;
}
void main()
{
    lowp vec4 source = texture2D(inputImageTexture, textureCoordinate);
    lowp vec4 blurred =  texture2D(inputImageTexture2, textureCoordinate);
    lowp vec4 destination =  texture2D(inputImageTexture3, textureCoordinate);
    lowp vec4 outa;

    if (destination.r > 0.0 || destination.g > 0.0 || destination.b > 0.0 || destination.a > 0.0){
        // If the layer has been ended while drawing
        if (source.rgba == vec4(0)){
            blurred *= 0.8;
            blurred *= destination.a;
            outa = blurred;
        }
        // Nowmal source color wrt destination
        else {
            source *= destination.a;
            source *= opacity;
            outa = source;
        }
    } else {
        outa = vec4(0.0);
    }
    if (drawing == 1){
        outa = vec4(mix(outa.rgb, monochromaticColor(outa).rgb, 1.0), outa.a);
    }
    gl_FragColor = outa;
}