precision highp float;

uniform sampler2D inputImageTexture;

uniform float imageWidth;
uniform float imageHeight;

uniform float strokeWidth;
uniform vec4 strokeColor;
uniform int disableStroke;

varying vec2 textureCoordinate;

void main(void)
{
    if (strokeWidth == 0. || disableStroke == 1) {
        gl_FragColor = texture2D(inputImageTexture, textureCoordinate);
        return;
    }

    vec2 size = vec2(1.0 / imageWidth, 1. / imageHeight) * strokeWidth;
    vec4 color = texture2D(inputImageTexture, textureCoordinate);

    float maxAlpha = 0.;
    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(-size.x, 0)).a);
    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(0, size.y)).a);
    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(size.x, 0)).a);
    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(0, -size.y)).a);

    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(-size.x, size.y)).a);
    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(size.x, size.y)).a);
    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(-size.x, -size.y)).a);
    maxAlpha = max(maxAlpha, texture2D(inputImageTexture, textureCoordinate + vec2(size.x, -size.y)).a);

    if (maxAlpha > color.a) {
        color = mix(color, strokeColor, maxAlpha - color.a);
    } else {
        color = vec4(0.);
    }

    gl_FragColor = color;
}