precision highp float;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform highp float clarity;

vec3 W = vec3(0.298,0.588,0.113);
vec3 rgb2hsv(vec3 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
    vec4 src = texture2D(inputImageTexture, textureCoordinate);
    float dClarity = clarity * 100.;
    float srcHum = dot(src.rgb,W);
    float avgerLum = srcHum;
    float weight = 1.0;
    vec2 inputSize2 = vec2(300, 300);
    for(float t=1.; t<=6.0; t++){
        float distance =t;
        float left = dot(texture2D(inputImageTexture, textureCoordinate +vec2(-distance/inputSize2.x ,0)).rgb,W);
        float right = dot(texture2D(inputImageTexture, textureCoordinate +vec2(distance/inputSize2.x ,0)).rgb,W);
        float top = dot(texture2D(inputImageTexture, textureCoordinate +vec2(0 ,-distance/inputSize2.y)).rgb,W);
        float bottom = dot(texture2D(inputImageTexture, textureCoordinate +vec2(0 ,distance/inputSize2.y)).rgb,W);
        float leftTop = dot(texture2D(inputImageTexture, textureCoordinate +vec2(-distance/inputSize2.x ,-distance/inputSize2.y)).rgb,W);
        float rightTop = dot(texture2D(inputImageTexture, textureCoordinate +vec2(distance/inputSize2.x ,-distance/inputSize2.y)).rgb,W);
        float leftBottom = dot(texture2D(inputImageTexture, textureCoordinate +vec2(-distance/inputSize2.x ,distance/inputSize2.y)).rgb,W);
        float rightBottom = dot(texture2D(inputImageTexture, textureCoordinate +vec2(distance/inputSize2.x ,distance/inputSize2.y)).rgb,W);
        float leftWeight = 1.0 - abs(left - srcHum);
        float rightWeight = 1.0 - abs(right - srcHum);
        float topWeight  = 1.0 - abs(top - srcHum);
        float bottomWeight = 1.0 - abs(bottom - srcHum);
        float leftTopWeight = 1.0 - abs(leftTop - srcHum);
        float righToptWeight = 1.0 - abs(rightTop - srcHum);
        float leftBottomWeight  = 1.0 - abs(leftBottom - srcHum);
        float rightBottomWeight = 1.0 - abs(rightBottom - srcHum);
        leftWeight = pow(leftWeight,8.0) ;
        rightWeight = pow(rightWeight,8.0) ;
        topWeight = pow(topWeight,8.0) ;
        bottomWeight = pow(bottomWeight,8.0) ;
        leftTopWeight = pow(leftTopWeight,8.0) ;
        righToptWeight = pow(righToptWeight,8.0) ;
        leftBottomWeight = pow(leftBottomWeight,8.0) ;
        rightBottomWeight = pow(rightBottomWeight,8.0) ;
        avgerLum = avgerLum + leftWeight * left + rightWeight *right + top * topWeight + bottom *bottomWeight+ leftTop *leftTopWeight + rightTop *righToptWeight + leftBottom *leftBottomWeight + rightBottom *rightBottomWeight ;
        weight = weight + leftWeight + rightWeight + topWeight + bottomWeight + leftTopWeight  +righToptWeight + leftBottomWeight + rightBottomWeight;
    }

    avgerLum = avgerLum/weight;
    vec3 srcHsv = rgb2hsv(src.rgb);
    float resultClarity;
    float dLum =srcHum - avgerLum;
    if(dClarity > 0.0){
        float dx =dLum > 0.0 ? 20.0 :15.0;
        gl_FragColor =vec4(src.rgb + src.rgb*(dClarity/dx *dLum ),src.a);
    } else{
        resultClarity = dClarity/34.56 ;
        srcHsv.z  = clamp(srcHsv.z *(1.0+ dLum * resultClarity *(1.2-srcHsv.z)), 0.0 , 1.0);
        gl_FragColor = vec4(hsv2rgb(srcHsv),src.a);
    }
}